pub struct Daylight {
pub sunrise: i32,
pub sunset: i32,
pub transition_duration: i32,
pub daylight_color: Vec3<f32>,
pub sunrise_color: Vec3<f32>,
pub sunset_color: Vec3<f32>,
pub night_color: Vec3<f32>,
}Fields
sunrise: i32sunset: i32transition_duration: i32daylight_color: Vec3<f32>sunrise_color: Vec3<f32>sunset_color: Vec3<f32>night_color: Vec3<f32>Implementations
Sourceimpl Daylight
impl Daylight
Sourcepub fn daylight(&self, time: i32, min_bright: f32, max_bright: f32) -> Vec3<f32>
pub fn daylight(&self, time: i32, min_bright: f32, max_bright: f32) -> Vec3<f32>
Computes the current sky color based on time, using XZ plane lighting.
Sourcepub fn daylight_intensity(&self, time: i32) -> f32
pub fn daylight_intensity(&self, time: i32) -> f32
Returns the intensity of daylight at a given time (0.0 = night, 1.0 = full daylight).
Sourcepub fn calculate_light_direction(&self, time: i32) -> Vec3<f32>
pub fn calculate_light_direction(&self, time: i32) -> Vec3<f32>
Computes the direction of the sun in the XZ plane based on the time of day.