pub struct Map {Show 19 fields
pub id: Uuid,
pub name: String,
pub offset: Vec2<f32>,
pub grid_size: f32,
pub subdivisions: f32,
pub possible_polygon: Vec<u32>,
pub curr_grid_pos: Option<Vec2<f32>>,
pub curr_mouse_pos: Option<Vec2<f32>>,
pub curr_rectangle: Option<(Vec2<f32>, Vec2<f32>)>,
pub vertices: Vec<Vertex>,
pub linedefs: Vec<Linedef>,
pub sectors: Vec<Sector>,
pub sky_texture: Option<Uuid>,
pub camera: MapCamera,
pub camera_xz: Option<Vec2<f32>>,
pub lights: Vec<Light>,
pub selected_vertices: Vec<u32>,
pub selected_linedefs: Vec<u32>,
pub selected_sectors: Vec<u32>,
}Fields
id: Uuidname: Stringoffset: Vec2<f32>grid_size: f32subdivisions: f32possible_polygon: Vec<u32>curr_grid_pos: Option<Vec2<f32>>curr_mouse_pos: Option<Vec2<f32>>curr_rectangle: Option<(Vec2<f32>, Vec2<f32>)>vertices: Vec<Vertex>linedefs: Vec<Linedef>sectors: Vec<Sector>sky_texture: Option<Uuid>camera: MapCameracamera_xz: Option<Vec2<f32>>lights: Vec<Light>selected_vertices: Vec<u32>selected_linedefs: Vec<u32>selected_sectors: Vec<u32>Implementations
Sourceimpl Map
impl Map
pub fn new() -> Self
pub fn clear_temp(&mut self)
Sourcepub fn bounding_box(&self) -> Option<Vec4<f32>>
pub fn bounding_box(&self) -> Option<Vec4<f32>>
Generate a bounding box for all vertices in the map
pub fn add_vertex_at(&mut self, x: f32, y: f32) -> u32
Sourcepub fn find_vertex_at(&self, x: f32, y: f32) -> Option<u32>
pub fn find_vertex_at(&self, x: f32, y: f32) -> Option<u32>
Finds a vertex at the given position and returns its ID if it exists
Sourcepub fn find_vertex(&self, id: u32) -> Option<&Vertex>
pub fn find_vertex(&self, id: u32) -> Option<&Vertex>
Finds a reference to a vertex by its ID
Sourcepub fn find_vertex_mut(&mut self, id: u32) -> Option<&mut Vertex>
pub fn find_vertex_mut(&mut self, id: u32) -> Option<&mut Vertex>
Finds a mutable reference to a vertex by its ID
Sourcepub fn find_linedef(&self, id: u32) -> Option<&Linedef>
pub fn find_linedef(&self, id: u32) -> Option<&Linedef>
Finds a reference to a linedef by its ID
Sourcepub fn find_linedef_mut(&mut self, id: u32) -> Option<&mut Linedef>
pub fn find_linedef_mut(&mut self, id: u32) -> Option<&mut Linedef>
Finds a reference to a linedef by its ID
Sourcepub fn find_sector(&self, id: u32) -> Option<&Sector>
pub fn find_sector(&self, id: u32) -> Option<&Sector>
Finds a mutable reference to a sector by its ID
Sourcepub fn find_sector_mut(&mut self, id: u32) -> Option<&mut Sector>
pub fn find_sector_mut(&mut self, id: u32) -> Option<&mut Sector>
Finds a mutable reference to a sector by its ID
pub fn create_linedef( &mut self, start_vertex: u32, end_vertex: u32, ) -> (u32, Option<u32>)
Sourcepub fn test_for_closed_polygon(&self) -> bool
pub fn test_for_closed_polygon(&self) -> bool
Check if the possible_polygon forms a closed loop
Sourcepub fn create_sector_from_polygon(&mut self) -> Option<u32>
pub fn create_sector_from_polygon(&mut self) -> Option<u32>
Tries to create a polyon from the tracked vertices in possible_polygon
Sourcepub fn delete_elements(
&mut self,
vertex_ids: &[u32],
linedef_ids: &[u32],
sector_ids: &[u32],
)
pub fn delete_elements( &mut self, vertex_ids: &[u32], linedef_ids: &[u32], sector_ids: &[u32], )
Deletes the specified vertices, linedefs, and sectors, along with their associated geometry.
Sourcepub fn is_linedef_in_closed_polygon(&self, linedef_id: u32) -> bool
pub fn is_linedef_in_closed_polygon(&self, linedef_id: u32) -> bool
Check if a given linedef ID is part of any sector.
Sourcepub fn add_to_selection(
&mut self,
vertices: Vec<u32>,
linedefs: Vec<u32>,
sectors: Vec<u32>,
)
pub fn add_to_selection( &mut self, vertices: Vec<u32>, linedefs: Vec<u32>, sectors: Vec<u32>, )
Add the given geometry to the selection.