pub struct Vertex {
pub id: u32,
pub name: String,
pub x: f32,
pub y: f32,
pub z: f32,
pub properties: ValueContainer,
}Fields
id: u32name: Stringx: f32y: f32z: f32properties: ValueContainerImplementations
Sourceimpl Vertex
impl Vertex
pub fn new(id: u32, x: f32, y: f32) -> Self
pub fn new_3d(id: u32, x: f32, y: f32, z: f32) -> Self
pub fn as_vec2(&self) -> Vec2<f32>
Sourcepub fn as_vec3_world(&self) -> Vec3<f32>
pub fn as_vec3_world(&self) -> Vec3<f32>
Returns a Vec3 in world coordinates, that means z is used as y-up.
Sourcepub fn intersects_vertical_slice(&self, slice_y: f32, thickness: f32) -> bool
pub fn intersects_vertical_slice(&self, slice_y: f32, thickness: f32) -> bool
Returns true if this vertex lies within a vertical slice centered at slice_y