Scene

Struct Scene 

pub struct Scene {
    pub background: Option<Box<dyn Shader>>,
    pub lights: Vec<Light>,
    pub d3_static: Vec<Batch<[f32; 4]>>,
    pub d3_dynamic: Vec<Batch<[f32; 4]>>,
    pub d2: Vec<Batch<[f32; 3]>>,
    pub textures: Vec<Texture>,
}
A scene of 2D and 3D batches which are passed to the rasterizer for rasterization.

Fields

background: Option<Box<dyn Shader>>lights: Vec<Light>d3_static: Vec<Batch<[f32; 4]>>

3D static batches which do not need to be changed, i.e. no animation for textures or the mesh itself.

d3_dynamic: Vec<Batch<[f32; 4]>>

3D dynamic batches which can be updated dynamically.

d2: Vec<Batch<[f32; 3]>>

The 2D batches get rendered on top of the 3D batches (2D game or UI).

textures: Vec<Texture>

The list of textures which the batches index into.

Implementations

Source

impl Scene

pub fn empty() -> Self

pub fn from_static(d2: Vec<Batch<[f32; 3]>>, d3: Vec<Batch<[f32; 4]>>) -> Self

Examples found in repository?
examples/cube.rs (lines 25-30)
21    fn new() -> Self
22    where
23        Self: Sized,
24    {
25        let scene = Scene::from_static(
26            vec![Batch::from_rectangle(0.0, 0.0, 200.0, 200.0)],
27            vec![Batch::from_box(-0.5, -0.5, -0.5, 1.0, 1.0, 1.0)
28                .sample_mode(SampleMode::Nearest)
29                .cull_mode(CullMode::Off)],
30        )
31        .background(Box::new(VGrayGradientShader::new()))
32        .textures(vec![Texture::from_image(Path::new("images/logo.png"))]);
33
34        Self {
35            camera: D3OrbitCamera::new(),
36            scene,
37        }
38    }
More examples
Hide additional examples
examples/obj.rs (lines 25-31)
21    fn new() -> Self
22    where
23        Self: Sized,
24    {
25        let scene = Scene::from_static(
26            vec![Batch::from_rectangle(0.0, 0.0, 200.0, 200.0)],
27            vec![Batch::from_obj(Path::new("examples/teapot.obj"))
28                .sample_mode(SampleMode::Linear)
29                .repeat_mode(RepeatMode::RepeatXY)
30                .transform(Mat4::scaling_3d(Vec3::new(0.35, -0.35, 0.35)))],
31        )
32        .background(Box::new(VGrayGradientShader::new()))
33        .textures(vec![Texture::from_image(Path::new("images/logo.png"))]);
34
35        Self {
36            camera: D3OrbitCamera::new(),
37            scene,
38        }
39    }
pub fn background(self, background: Box<dyn Shader>) -> Self

Sets the background shader using the builder pattern.

Examples found in repository?
examples/cube.rs (line 31)
21    fn new() -> Self
22    where
23        Self: Sized,
24    {
25        let scene = Scene::from_static(
26            vec![Batch::from_rectangle(0.0, 0.0, 200.0, 200.0)],
27            vec![Batch::from_box(-0.5, -0.5, -0.5, 1.0, 1.0, 1.0)
28                .sample_mode(SampleMode::Nearest)
29                .cull_mode(CullMode::Off)],
30        )
31        .background(Box::new(VGrayGradientShader::new()))
32        .textures(vec![Texture::from_image(Path::new("images/logo.png"))]);
33
34        Self {
35            camera: D3OrbitCamera::new(),
36            scene,
37        }
38    }
More examples
Hide additional examples
examples/obj.rs (line 32)
21    fn new() -> Self
22    where
23        Self: Sized,
24    {
25        let scene = Scene::from_static(
26            vec![Batch::from_rectangle(0.0, 0.0, 200.0, 200.0)],
27            vec![Batch::from_obj(Path::new("examples/teapot.obj"))
28                .sample_mode(SampleMode::Linear)
29                .repeat_mode(RepeatMode::RepeatXY)
30                .transform(Mat4::scaling_3d(Vec3::new(0.35, -0.35, 0.35)))],
31        )
32        .background(Box::new(VGrayGradientShader::new()))
33        .textures(vec![Texture::from_image(Path::new("images/logo.png"))]);
34
35        Self {
36            camera: D3OrbitCamera::new(),
37            scene,
38        }
39    }
pub fn textures(self, textures: Vec<Texture>) -> Self

Sets the background shader using the builder pattern.

Examples found in repository?
examples/cube.rs (line 32)
21    fn new() -> Self
22    where
23        Self: Sized,
24    {
25        let scene = Scene::from_static(
26            vec![Batch::from_rectangle(0.0, 0.0, 200.0, 200.0)],
27            vec![Batch::from_box(-0.5, -0.5, -0.5, 1.0, 1.0, 1.0)
28                .sample_mode(SampleMode::Nearest)
29                .cull_mode(CullMode::Off)],
30        )
31        .background(Box::new(VGrayGradientShader::new()))
32        .textures(vec![Texture::from_image(Path::new("images/logo.png"))]);
33
34        Self {
35            camera: D3OrbitCamera::new(),
36            scene,
37        }
38    }
More examples
Hide additional examples
examples/obj.rs (line 33)
21    fn new() -> Self
22    where
23        Self: Sized,
24    {
25        let scene = Scene::from_static(
26            vec![Batch::from_rectangle(0.0, 0.0, 200.0, 200.0)],
27            vec![Batch::from_obj(Path::new("examples/teapot.obj"))
28                .sample_mode(SampleMode::Linear)
29                .repeat_mode(RepeatMode::RepeatXY)
30                .transform(Mat4::scaling_3d(Vec3::new(0.35, -0.35, 0.35)))],
31        )
32        .background(Box::new(VGrayGradientShader::new()))
33        .textures(vec![Texture::from_image(Path::new("images/logo.png"))]);
34
35        Self {
36            camera: D3OrbitCamera::new(),
37            scene,
38        }
39    }
pub fn lights(self, lights: Vec<Light>) -> Self

Sets the lights using the builder pattern.

pub fn project( &mut self, projection_matrix_2d: Option<Mat3<f32>>, view_matrix_3d: Mat4<f32>, projection_matrix_3d: Mat4<f32>, width: usize, height: usize, )

Project the batches using the given matrices (which represent the global camera).