pub struct Item {
pub id: u32,
pub position: Vec3<f32>,
pub creator_id: Uuid,
pub item_type: String,
pub max_capacity: u32,
pub container: Option<Vec<Item>>,
pub attributes: ValueContainer,
pub dirty_flags: u8,
pub dirty_attributes: FxHashSet<String>,
}Fields
id: u32The unique ID of the item
position: Vec3<f32>The position in the map (if not on an entity).
creator_id: UuidMaps the item to a creator ID
item_type: StringThe item’s type or identifier (e.g., “Potion”, “Sword”)
max_capacity: u32Maximum capacity of this container/stack (e.g., max stack size for stackable items)
container: Option<Vec<Item>>Container: Holds nested items if this item can act as a container
attributes: ValueContainerAttributes: Dynamic properties of the item
dirty_flags: u8Dirty flags for static attributes
dirty_attributes: FxHashSet<String>Dirty dynamic attributes
Implementations
Sourceimpl Item
impl Item
pub fn new() -> Self
Sourcepub fn get_pos_xz(&self) -> Vec2<f32>
pub fn get_pos_xz(&self) -> Vec2<f32>
Get the XZ position.
Sourcepub fn set_position(&mut self, new_position: Vec3<f32>)
pub fn set_position(&mut self, new_position: Vec3<f32>)
Set the position and mark it as dirty
Sourcepub fn set_max_capacity(&mut self, new_max_capacity: u32)
pub fn set_max_capacity(&mut self, new_max_capacity: u32)
Set the position and mark it as dirty
Sourcepub fn is_container(&self) -> bool
pub fn is_container(&self) -> bool
Check if the item is a container or stackable
Sourcepub fn add_to_container(&mut self, item: Item) -> Result<(), String>
pub fn add_to_container(&mut self, item: Item) -> Result<(), String>
Add an item to the container
Sourcepub fn remove_from_container(&mut self, item_id: u32) -> Result<Item, String>
pub fn remove_from_container(&mut self, item_id: u32) -> Result<Item, String>
Remove an item from the container by ID
Sourcepub fn set_attribute(&mut self, key: &str, value: Value)
pub fn set_attribute(&mut self, key: &str, value: Value)
Set a dynamic attribute and mark it as dirty
Sourcepub fn get_attribute(&self, key: &str) -> Option<&Value>
pub fn get_attribute(&self, key: &str) -> Option<&Value>
Get a dynamic attribute
Sourcepub fn get_attr_string(&self, key: &str) -> Option<String>
pub fn get_attr_string(&self, key: &str) -> Option<String>
Get the given String
Sourcepub fn get_attr_uuid(&self, key: &str) -> Option<Uuid>
pub fn get_attr_uuid(&self, key: &str) -> Option<Uuid>
Get the given Uuid
Sourcepub fn mark_dirty_attribute(&mut self, key: &str)
pub fn mark_dirty_attribute(&mut self, key: &str)
Mark a dynamic attribute as dirty
Sourcepub fn mark_all_dirty(&mut self)
pub fn mark_all_dirty(&mut self)
Mark all fields and attributes as dirty
Sourcepub fn clear_dirty(&mut self)
pub fn clear_dirty(&mut self)
Clear all dirty flags and attributes
Sourcepub fn get_update(&self) -> ItemUpdate
pub fn get_update(&self) -> ItemUpdate
Generate an ItemUpdate containing only dirty fields and attributes
Sourcepub fn apply_update(&mut self, update: ItemUpdate)
pub fn apply_update(&mut self, update: ItemUpdate)
Apply an ItemUpdate to this item