pub struct RenderBuffer {
pub width: usize,
pub height: usize,
pub pixels: Vec<f32>,
pub accum: u32,
pub file_path: Option<PathBuf>,
}Expand description
A color buffer holding an array of float pixels.
Fields
width: usizeheight: usizepixels: Vec<f32>accum: u32file_path: Option<PathBuf>Implementations
Sourceimpl RenderBuffer
impl RenderBuffer
pub fn new(width: usize, height: usize) -> Self
Sourcepub fn copy_from(&mut self, x: usize, y: usize, other: &RenderBuffer)
pub fn copy_from(&mut self, x: usize, y: usize, other: &RenderBuffer)
Copy the pixels from another buffer to this buffer
Sourcepub fn accum_from(&mut self, x: usize, y: usize, other: &RenderBuffer)
pub fn accum_from(&mut self, x: usize, y: usize, other: &RenderBuffer)
Copy and accumulate pixels from another buffer to this buffer
Sourcepub fn to_u8_vec_gamma(&self) -> Vec<u8> ⓘ
pub fn to_u8_vec_gamma(&self) -> Vec<u8> ⓘ
Convert the frame to an u8 vec, applying gamma correction
Sourcepub fn to_u8_vec_gamma_buffer(&self, buffer: &mut [u8])
pub fn to_u8_vec_gamma_buffer(&self, buffer: &mut [u8])
Convert the frame to an u8 vec, applying gamma correction
Sourcepub fn as_rgba_bytes(&self) -> Vec<u8> ⓘ
pub fn as_rgba_bytes(&self) -> Vec<u8> ⓘ
Return raw RGBA8 bytes suitable for JS ImageData (sRGB gamma corrected).
Length is width * height * 4.
Sourcepub fn write_rgba_to_buffer(&self, out: &mut [u8])
pub fn write_rgba_to_buffer(&self, out: &mut [u8])
Fill the provided buffer with raw RGBA8 bytes (sRGB gamma corrected).
out must be length width * height * 4.
pub fn save_srgb(&self, path: PathBuf)
pub fn save_film(&self, path: PathBuf)
pub fn as_png_bytes(&self) -> Vec<u8> ⓘ